/**
 * @file	UniformDiscreteMeasure.java
 * 
 * @date	Feb 8, 2005
 * @author	Alexandre Bouchard
 */
package ca.mcgill.lmp.util.measures;


import java.util.Iterator;
import java.util.Random;

import ca.mcgill.lmp.EmptySetException;
import ca.mcgill.lmp.MeasurableSet;
import ca.mcgill.lmp.Measure;
import ca.mcgill.lmp.State;
import ca.mcgill.lmp.approx.MonteCarlo;
import ca.mcgill.lmp.util.DiscreteSet;

/**
 * 
 *
 */
public class UniformDiscreteMeasure implements Measure
{
    /**
     * Comment for <code>states</code>
     */
    private DiscreteSet states;
    private State[] statesList;
    
    /**
     * Comment for <code>rand</code>
     */
    private Random rand = new Random(System.currentTimeMillis());
    
    /**
     * @param states
     */
    public UniformDiscreteMeasure(DiscreteSet states)
    {
        this.states = states;
        statesList = new State[states.size()];
        Iterator it = states.iterator();
        int i = 0;
        while (it.hasNext())
        {
            State currentState = (State) it.next();
            statesList[i] = currentState;
            i++;
        }
    }

    /**
     * @param B
     * @return
     */
    public double measure(MeasurableSet B)
    {
        DiscreteSet set = (DiscreteSet) B;
        return ((double) set.size())/((double) states.size());
    }

    /**
     * @param B
     * @return
     */
    public State sampleIn(MeasurableSet B) throws EmptySetException
    {
        State currentState = statesList[rand.nextInt(states.size())];
        for (int i = 0; i < MonteCarlo.DEFAULT_NUMBER_OF_TRIALS; i++)
        {
            if (B.contains(currentState))
            {
                return currentState;
            }
            currentState = statesList[rand.nextInt(states.size())];
        }
        throw new EmptySetException();
    }

    /**
     * @return
     */
    public MeasurableSet getX()
    {
        return states;
    }
    
    public String toString()
    {
        return "Discrete distribution on " + states.toString();
    }
    
    /**
     * @return
     */
    public State sample()
    {
        try
        {
            return sampleIn(getX());
        }
        catch (EmptySetException ese)
        {
            System.out.println(ese.toString());
            return null;
        }
    }    
}
